@@ -13,11 +13,11 @@ pub struct MethodParam {
 }
 
 impl MethodSignature {
-    pub fn dependencies(&self) -> Vec<ElementType> {
+    pub fn dependencies(&self, include: TypeInclude) -> Vec<TypeEntry> {
         self.return_type
             .iter()
-            .map(|s| s.definition())
-            .chain(self.params.iter().map(|p| p.signature.definition()))
+            .map(|s| s.definition(include))
+            .chain(self.params.iter().map(|p| p.signature.definition(include)))
             .flatten()
             .collect()
     }
@@ -41,6 +41,38 @@ impl MethodSignature {
         })
     }
 
+    pub fn has_retval(&self) -> bool {
+        self.return_type.as_ref().map_or(false, |signature| {
+            if signature.kind == ElementType::HRESULT
+                && self.params.last().map_or(false, |param| {
+                    let flags = param.param.flags();
+                    if flags.input() || !flags.output() || param.signature.pointers != 1 {
+                        return false;
+                    }
+
+                    match &param.signature.kind {
+                        ElementType::Void => false,
+                        ElementType::TypeDef(def) => def.kind() != TypeKind::Delegate,
+                        _ => true,
+                    }
+                })
+            {
+                return self.params[..self.params.len() - 1].iter().all(|param| {
+                    let flags = param.param.flags();
+                    flags.input() && !flags.output()
+                });
+            }
+
+            false
+        })
+    }
+
+    pub fn has_udt_return(&self) -> bool {
+        self.return_type
+            .as_ref()
+            .map_or(false, |signature| signature.is_udt())
+    }
+
     pub fn gen_winrt_constraint(&self, gen: &Gen) -> TokenStream {
         let params = self.params.iter().map(|p| p.gen_winrt_produce_type(gen));
 
@@ -62,10 +94,6 @@ impl MethodSignature {
     // All WinRT ABI methods return an HRESULT while any return type is transformed into a trailing
     // out parameter. This is unlike Win32 methods that don't require this transformation.
     pub fn gen_winrt_abi(&self, gen: &Gen) -> TokenStream {
-        if !gen.include_method(self) {
-            return quote! { () };
-        }
-
         let params = self
             .params
             .iter()
@@ -114,10 +142,6 @@ impl MethodSignature {
         interface: &InterfaceInfo,
         gen: &Gen,
     ) -> TokenStream {
-        if !gen.include_method(self) {
-            return quote! {};
-        }
-
         let params = if interface.kind == InterfaceKind::Composable
             || interface.kind == InterfaceKind::Extend
         {
